-- Test add_media
set client_min_messages to warning;
create extension if not exists pgtap;
reset client_min_messages;

begin;

set search_path to camper, public;

select plan(13);

select has_function('camper', 'add_media', array ['integer', 'text', 'media_type', 'bytea']);
select function_lang_is('camper', 'add_media', array ['integer', 'text', 'media_type', 'bytea'], 'plpgsql');
select function_returns('camper', 'add_media', array ['integer', 'text', 'media_type', 'bytea'], 'integer');
select isnt_definer('camper', 'add_media', array ['integer', 'text', 'media_type', 'bytea']);
select volatility_is('camper', 'add_media', array ['integer', 'text', 'media_type', 'bytea'], 'volatile');
select function_privs_are('camper', 'add_media', array ['integer', 'text', 'media_type', 'bytea'], 'guest', array[]::text[]);
select function_privs_are('camper', 'add_media', array ['integer', 'text', 'media_type', 'bytea'], 'employee', array[]::text[]);
select function_privs_are('camper', 'add_media', array ['integer', 'text', 'media_type', 'bytea'], 'admin', array['EXECUTE']);
select function_privs_are('camper', 'add_media', array ['integer', 'text', 'media_type', 'bytea'], 'authenticator', array[]::text[]);


set client_min_messages to warning;
truncate media cascade;
truncate media_content cascade;
truncate company cascade;
reset client_min_messages;

insert into company (company_id, business_name, vatin, trade_name, phone, email, web, address, city, province, postal_code, country_code, currency_code, default_lang_tag)
values (1, 'Company 2', 'XX123', '', '555-555-555', 'a@a', '', '', '', '', '', 'ES', 'EUR', 'ca')
     , (2, 'Company 4', 'XX234', '', '666-666-666', 'b@b', '', '', '', '', '', 'FR', 'USD', 'ca')
;

select lives_ok(
	$$ select add_media(1, 'text.txt', 'text/plain', 'hello, world!') $$,
	'Should be able to add a media to the first company'
);

select lives_ok(
	$$ select add_media(2, 'image.svg', 'image/svg+xml', '<svg xmlns="http://www.w3.org/2000/svg" width="1" height="1"/>') $$,
	'Should be able to add a media to the second company'
);

select lives_ok(
	$$ select add_media(2, 'world.txt', 'text/plain', 'hello, world!') $$,
	'Should be able to add a media to the second company with the same content as the file from the first company'
);

select bag_eq(
	$$ select company_id, content_hash, original_filename, media_type, convert_from(bytes, 'utf-8') from media join media_content using (content_hash) $$,
	$$ values (1, sha256('hello, world!'), 'text.txt', 'text/plain', 'hello, world!')
	        , (2, sha256('<svg xmlns="http://www.w3.org/2000/svg" width="1" height="1"/>'), 'image.svg', 'image/svg+xml', '<svg xmlns="http://www.w3.org/2000/svg" width="1" height="1"/>')
	        , (2, sha256('hello, world!'), 'world.txt', 'text/plain', 'hello, world!')
	$$,
	'Should have added all three media'
);

select *
from finish();

rollback;
