-- Test build_cookie
set client_min_messages to warning;
create extension if not exists pgtap;
reset client_min_messages;

begin;

select plan(13);

set search_path to camper, auth, public;

select has_function('camper', 'build_cookie', array ['email', 'text']);
select function_lang_is('camper', 'build_cookie', array ['email', 'text'], 'sql');
select function_returns('camper', 'build_cookie', array ['email', 'text'], 'text');
select isnt_definer('camper', 'build_cookie', array ['email', 'text']);
select volatility_is('camper', 'build_cookie', array ['email', 'text'], 'stable');
select function_privs_are('camper', 'build_cookie', array ['email', 'text'], 'guest', array []::text[]);
select function_privs_are('camper', 'build_cookie', array ['email', 'text'], 'employee', array ['EXECUTE']);
select function_privs_are('camper', 'build_cookie', array ['email', 'text'], 'admin', array ['EXECUTE']);
select function_privs_are('camper', 'build_cookie', array ['email', 'text'], 'authenticator', array []::text[]);

set client_min_messages to warning;
truncate company_host cascade;
truncate company_user cascade;
truncate company cascade;
truncate auth."user" cascade;
reset client_min_messages;

insert into auth."user" (user_id, email, name, password, cookie, cookie_expires_at)
values (1, 'demo@tandem.blog', 'Demo', 'test', '44facbb30d8a419dfd4bfbc44a4b5539d4970148dfc84bed0e', current_timestamp + interval '1 month')
     , (9, 'admin@tandem.blog', 'Demo', 'test', '12af4c88b528c2ad4222e3740496ecbc58e76e26f087657524', current_timestamp + interval '1 month')
;

insert into company (company_id, business_name, vatin, trade_name, phone, email, web, address, city, province, postal_code, country_code, currency_code, default_lang_tag)
values (2, 'Company 2', 'XX123', '', '555-555-555', 'a@a', '', '', '', '', '', 'ES', 'EUR', 'ca')
;

insert into company_user (company_id, user_id, role)
values (2, 1, 'employee')
     , (2, 9, 'admin')
;

insert into company_host (company_id, host)
values (2, 'localhost')
;

select is(
	build_cookie('test@example.com'::email, '123abc'),
	'123abc/test@example.com',
	'Should build the cookie with the given user and cookie value'
);

select set_cookie('44facbb30d8a419dfd4bfbc44a4b5539d4970148dfc84bed0e/demo@tandem.blog', 'localhost');
reset role;

select is(
	build_cookie(),
	'44facbb30d8a419dfd4bfbc44a4b5539d4970148dfc84bed0e/demo@tandem.blog',
	'Should build the cookie for the logged in user'
);


select set_cookie('12af4c88b528c2ad4222e3740496ecbc58e76e26f087657524/admin@tandem.blog', 'localhost');
reset role;

select is(
	build_cookie(),
	'12af4c88b528c2ad4222e3740496ecbc58e76e26f087657524/admin@tandem.blog',
	'Should build the cookie for the other logged in user'
);


select set_cookie('ashtasth', 'localhost');
reset role;

select is(
	build_cookie(),
	'/',
	'Should build the cookie for the guest user'
);


select *
from finish();

rollback;
