-- Test campsite
set client_min_messages to warning;
create extension if not exists pgtap;
reset client_min_messages;

begin;

select plan(47);

set search_path to camper, public;

select has_table('campsite');
select has_pk('campsite');
select col_is_unique('campsite', array['company_id', 'label']);
select table_privs_are('campsite', 'guest', array['SELECT']);
select table_privs_are('campsite', 'employee', array['SELECT']);
select table_privs_are('campsite', 'admin', array['SELECT', 'INSERT', 'UPDATE', 'DELETE']);
select table_privs_are('campsite', 'authenticator', array[]::text[]);

select has_column('campsite', 'campsite_id');
select col_is_pk('campsite', 'campsite_id');
select col_type_is('campsite', 'campsite_id', 'integer');
select col_not_null('campsite', 'campsite_id');
select col_hasnt_default('campsite', 'campsite_id');

select has_column('campsite', 'company_id');
select col_is_fk('campsite', 'company_id');
select fk_ok('campsite', 'company_id', 'company', 'company_id');
select col_type_is('campsite', 'company_id', 'integer');
select col_not_null('campsite', 'company_id');
select col_hasnt_default('campsite', 'company_id');

select has_column('campsite', 'campsite_type_id');
select col_is_fk('campsite', 'campsite_type_id');
select fk_ok('campsite', 'campsite_type_id', 'campsite_type', 'campsite_type_id');
select col_type_is('campsite', 'campsite_type_id', 'integer');
select col_not_null('campsite', 'campsite_type_id');
select col_hasnt_default('campsite', 'campsite_type_id');

select has_column('campsite', 'label');
select col_type_is('campsite', 'label', 'text');
select col_not_null('campsite', 'label');
select col_hasnt_default('campsite', 'label');

select has_column('campsite', 'active');
select col_type_is('campsite', 'active', 'boolean');
select col_not_null('campsite', 'active');
select col_has_default('campsite', 'active');
select col_default_is('campsite', 'active', 'true');


set client_min_messages to warning;
truncate campsite cascade;
truncate campsite_type cascade;
truncate media cascade;
truncate media_content cascade;
truncate company_host cascade;
truncate company_user cascade;
truncate company cascade;
truncate auth."user" cascade;
reset client_min_messages;


insert into auth."user" (user_id, email, name, password, cookie, cookie_expires_at)
values (1, 'demo@tandem.blog', 'Demo', 'test', '44facbb30d8a419dfd4bfbc44a4b5539d4970148dfc84bed0e', current_timestamp + interval '1 month')
     , (5, 'admin@tandem.blog', 'Demo', 'test', '12af4c88b528c2ad4222e3740496ecbc58e76e26f087657524', current_timestamp + interval '1 month')
;

insert into company (company_id, business_name, vatin, trade_name, phone, email, web, address, city, province, postal_code, country_code, currency_code, default_lang_tag)
values (2, 'Company 2', 'XX123', '', '555-555-555', 'a@a', '', '', '', '', '', 'ES', 'EUR', 'ca')
     , (4, 'Company 4', 'XX234', '', '666-666-666', 'b@b', '', '', '', '', '', 'FR', 'USD', 'ca')
;

insert into company_user (company_id, user_id, role)
values (2, 1, 'admin')
     , (4, 5, 'admin')
;

insert into company_host (company_id, host)
values (2, 'co2')
     , (4, 'co4')
;

insert into media_content (media_type, bytes)
values ('image/x-xpixmap', 'static char *s[]={"1 1 1 1","a c #ffffff","a"};')
;

insert into media (media_id, company_id, original_filename, content_hash)
values (6, 2, 'cover2.xpm', sha256('static char *s[]={"1 1 1 1","a c #ffffff","a"};'))
     , (8, 4, 'cover4.xpm', sha256('static char *s[]={"1 1 1 1","a c #ffffff","a"};'))
;

insert into campsite_type (campsite_type_id, company_id, media_id, name, dogs_allowed, max_campers)
values (22, 2, 6, 'Wooden lodge', true, 5)
     , (44, 4, 8, 'Bungalow', false, 4)
;

insert into campsite (company_id, campsite_type_id, label)
values (2, 22, 'W1')
     , (4, 44, 'B1')
;

prepare campsite_data as
select company_id, label
from campsite
order by company_id, label;

set role guest;
select bag_eq(
	'campsite_data',
	$$ values (2, 'W1')
	        , (4, 'B1')
	$$,
	'Everyone should be able to list all campsites across all companies'
);
reset role;

select set_cookie('44facbb30d8a419dfd4bfbc44a4b5539d4970148dfc84bed0e/demo@tandem.blog', 'co2');

select lives_ok(
	$$ insert into campsite(company_id, campsite_type_id, label) values (2, 22, 'w2' ) $$,
	'Admin from company 2 should be able to insert a new campsite to that company.'
);

select bag_eq(
	'campsite_data',
	$$ values (2, 'W1')
	        , (2, 'w2')
	        , (4, 'B1')
	$$,
	'The new row should have been added'
);

select lives_ok(
	$$ update campsite set label = 'W2' where company_id = 2 and label = 'w2' $$,
	'Admin from company 2 should be able to update campsites of that company.'
);

select bag_eq(
	'campsite_data',
	$$ values (2, 'W1')
	        , (2, 'W2')
	        , (4, 'B1')
	$$,
	'The row should have been updated.'
);

select lives_ok(
	$$ delete from campsite where company_id = 2 and label = 'W2' $$,
	'Admin from company 2 should be able to delete campsites from that company.'
);

select bag_eq(
	'campsite_data',
	$$ values (2, 'W1')
	        , (4, 'B1')
	$$,
	'The row should have been deleted.'
);

select throws_ok(
	$$ insert into campsite (company_id, campsite_type_id, label) values (4, 44, 'W3' ) $$,
	'42501', 'new row violates row-level security policy for table "campsite"',
	'Admin from company 2 should NOT be able to insert new campsites to company 4.'
);

select lives_ok(
	$$ update campsite set label = 'Nope' where company_id = 4 $$,
	'Admin from company 2 should NOT be able to update campsite types of company 4, but no error if company_id is not changed.'
);

select bag_eq(
	'campsite_data',
	$$ values (2, 'W1')
	        , (4, 'B1')
	$$,
	'No row should have been changed.'
);

select throws_ok(
	$$ update campsite set company_id = 4 where company_id = 2 $$,
	'42501', 'new row violates row-level security policy for table "campsite"',
	'Admin from company 2 should NOT be able to move campsites to company 4'
);

select lives_ok(
	$$ delete from campsite where company_id = 4 $$,
	'Admin from company 2 should NOT be able to delete campsite types from company 4, but not error is thrown'
);

select bag_eq(
	'campsite_data',
	$$ values (2, 'W1')
	        , (4, 'B1')
	$$,
	'No row should have been changed'
);

select throws_ok(
	$$ insert into campsite (company_id, campsite_type_id, label) values (2, 22, '  ' ) $$,
	'23514', 'new row for relation "campsite" violates check constraint "label_not_empty"',
	'Should not be able to insert campsites with a blank label.'
);

reset role;

select *
from finish();

rollback;

