-- Test campsite_type_cost
set client_min_messages to warning;
create extension if not exists pgtap;
reset client_min_messages;

begin;

select plan(47);

set search_path to camper, public;

select has_table('campsite_type_cost');
select has_pk('campsite_type_cost');
select col_is_pk('campsite_type_cost', array['campsite_type_id', 'season_id']);
select table_privs_are('campsite_type_cost', 'guest', array['SELECT']);
select table_privs_are('campsite_type_cost', 'employee', array['SELECT']);
select table_privs_are('campsite_type_cost', 'admin', array['SELECT', 'INSERT', 'UPDATE', 'DELETE']);
select table_privs_are('campsite_type_cost', 'authenticator', array[]::text[]);

select has_column('campsite_type_cost', 'campsite_type_id');
select col_is_fk('campsite_type_cost', 'campsite_type_id');
select fk_ok('campsite_type_cost', 'campsite_type_id', 'campsite_type', 'campsite_type_id');
select col_type_is('campsite_type_cost', 'campsite_type_id', 'integer');
select col_not_null('campsite_type_cost', 'campsite_type_id');
select col_hasnt_default('campsite_type_cost', 'campsite_type_id');

select has_column('campsite_type_cost', 'season_id');
select col_is_fk('campsite_type_cost', 'season_id');
select fk_ok('campsite_type_cost', 'season_id', 'season', 'season_id');
select col_type_is('campsite_type_cost', 'season_id', 'integer');
select col_not_null('campsite_type_cost', 'season_id');
select col_hasnt_default('campsite_type_cost', 'season_id');

select has_column('campsite_type_cost', 'cost_per_night');
select col_type_is('campsite_type_cost', 'cost_per_night', 'integer');
select col_not_null('campsite_type_cost', 'cost_per_night');
select col_hasnt_default('campsite_type_cost', 'cost_per_night');

select has_column('campsite_type_cost', 'min_nights');
select col_type_is('campsite_type_cost', 'min_nights', 'integer');
select col_not_null('campsite_type_cost', 'min_nights');
select col_hasnt_default('campsite_type_cost', 'min_nights');


set client_min_messages to warning;
truncate campsite_type_cost cascade;
truncate season cascade;
truncate campsite_type cascade;
truncate media cascade;
truncate media_content cascade;
truncate company_host cascade;
truncate company_user cascade;
truncate company cascade;
truncate auth."user" cascade;
reset client_min_messages;

insert into auth."user" (user_id, email, name, password, cookie, cookie_expires_at)
values (1, 'demo@tandem.blog', 'Demo', 'test', '44facbb30d8a419dfd4bfbc44a4b5539d4970148dfc84bed0e', current_timestamp + interval '1 month')
     , (5, 'admin@tandem.blog', 'Demo', 'test', '12af4c88b528c2ad4222e3740496ecbc58e76e26f087657524', current_timestamp + interval '1 month')
;

insert into company (company_id, business_name, vatin, trade_name, phone, email, web, address, city, province, postal_code, country_code, currency_code, default_lang_tag)
values (2, 'Company 2', 'XX123', '', '555-555-555', 'a@a', '', '', '', '', '', 'ES', 'EUR', 'ca')
     , (4, 'Company 4', 'XX234', '', '666-666-666', 'b@b', '', '', '', '', '', 'FR', 'USD', 'ca')
;

insert into company_user (company_id, user_id, role)
values (2, 1, 'admin')
     , (4, 5, 'admin')
;

insert into company_host (company_id, host)
values (2, 'co2')
     , (4, 'co4')
;

insert into media_content (media_type, bytes)
values ('image/x-xpixmap', 'static char *s[]={"1 1 1 1","a c #ffffff","a"};')
;

insert into media (media_id, company_id, original_filename, content_hash)
values (6, 2, 'cover2.xpm', sha256('static char *s[]={"1 1 1 1","a c #ffffff","a"};'))
     , (8, 4, 'cover4.xpm', sha256('static char *s[]={"1 1 1 1","a c #ffffff","a"};'))
;

insert into campsite_type (campsite_type_id, company_id, name, media_id, dogs_allowed, max_campers)
values (16, 2, 'Wooden lodge', 6, false, 7)
     , (18, 4, 'Bungalow', 8, false, 6)
;

insert into season (season_id, company_id, name)
values (26, 2, 'Low')
     , (27, 2, 'Mid')
     , (28, 4, 'Low')
     , (29, 4, 'Mid')
;

insert into campsite_type_cost (campsite_type_id, season_id, cost_per_night, min_nights)
values (16, 26, 2, 2)
     , (18, 28, 4, 4)
;

prepare campsite_season_data as
select campsite_type_id, season_id, cost_per_night
from campsite_type_cost
;

set role guest;
select bag_eq(
	'campsite_season_data',
	$$ values (16, 26, 2)
	        , (18, 28, 4)
	$$,
	'Everyone should be able to list all campsite type costs across all companies'
);
reset role;

select set_cookie('44facbb30d8a419dfd4bfbc44a4b5539d4970148dfc84bed0e/demo@tandem.blog', 'co2');

select lives_ok(
	$$ insert into campsite_type_cost(campsite_type_id, season_id, cost_per_night, min_nights) values (16, 27, 3, 3) $$,
	'Admin from company 2 should be able to insert a new campsite type season to that company.'
);

select bag_eq(
	'campsite_season_data',
	$$ values (16, 26, 2)
	        , (16, 27, 3)
	        , (18, 28, 4)
	$$,
	'The new row should have been added'
);

select lives_ok(
	$$ update campsite_type_cost set cost_per_night = 6 where campsite_type_id = 16 and season_id = 27 $$,
	'Admin from company 2 should be able to update campsite type season of that company.'
);

select bag_eq(
	'campsite_season_data',
	$$ values (16, 26, 2)
	        , (16, 27, 6)
	        , (18, 28, 4)
	$$,
	'The row should have been updated.'
);

select lives_ok(
	$$ delete from campsite_type_cost where campsite_type_id = 16 and season_id = 27 $$,
	'Admin from company 2 should be able to delete campsite type season from that company.'
);

select bag_eq(
	'campsite_season_data',
	$$ values (16, 26, 2)
	        , (18, 28, 4)
	$$,
	'The row should have been deleted.'
);

select throws_ok(
	$$ insert into campsite_type_cost (campsite_type_id, season_id, cost_per_night, min_nights) values (18, 29, 5, 5) $$,
	'42501', 'new row violates row-level security policy for table "campsite_type_cost"',
	'Admin from company 2 should NOT be able to insert new campsite type costs to company 4.'
);

select throws_ok(
	$$ insert into campsite_type_cost (campsite_type_id, season_id, cost_per_night, min_nights) values (18, 27, 5, 5) $$,
	'42501', 'new row violates row-level security policy for table "campsite_type_cost"',
	'Admin from company 2 should NOT be able to insert new row with a campsite type from company 4.'
);

select throws_ok(
	$$ insert into campsite_type_cost (campsite_type_id, season_id, cost_per_night, min_nights) values (16, 29, 5, 5) $$,
	'42501', 'new row violates row-level security policy for table "campsite_type_cost"',
	'Admin from company 2 should NOT be able to insert new row with a season from company 4.'
);

select lives_ok(
	$$ update campsite_type_cost set cost_per_night = 1 where campsite_type_id = 18 $$,
	'Admin from company 2 should not be able to update campsite types of company 4, but no error if campsite_type_id is not changed.'
);

select lives_ok(
	$$ update campsite_type_cost set cost_per_night = 1 where season_id = 28 $$,
	'Admin from company 2 should not be able to update seasons of company 4, but no error if season_id is not changed.'
);

select bag_eq(
	'campsite_season_data',
	$$ values (16, 26, 2)
	        , (18, 28, 4)
	$$,
	'No row should have been changed.'
);

select throws_ok(
	$$ update campsite_type_cost set campsite_type_id = 18 where campsite_type_id = 16 $$,
	'42501', 'new row violates row-level security policy for table "campsite_type_cost"',
	'Admin from company 2 should NOT be able to move campsite type to one of company 4'
);

select throws_ok(
	$$ update campsite_type_cost set season_id = 29 where season_id = 26 $$,
	'42501', 'new row violates row-level security policy for table "campsite_type_cost"',
	'Admin from company 2 should NOT be able to move season to one of company 4'
);

select lives_ok(
	$$ delete from campsite_type_cost where campsite_type_id = 18 $$,
	'Admin from company 2 should NOT be able to delete campsite type from company 4, but not error is thrown'
);

select lives_ok(
	$$ delete from campsite_type_cost where season_id = 28 $$,
	'Admin from company 2 should NOT be able to delete season from company 4, but not error is thrown'
);

select bag_eq(
	'campsite_season_data',
	$$ values (16, 26, 2)
	        , (18, 28, 4)
	$$,
	'No row should have been changed'
);

select throws_ok(
	$$ insert into campsite_type_cost (campsite_type_id, season_id, cost_per_night, min_nights) values (16, 27, -1, 1) $$,
	'23514', 'new row for relation "campsite_type_cost" violates check constraint "cost_not_negative"',
	'Should not be able to insert campsite type costs with negative cost per night.'
);

select throws_ok(
	$$ insert into campsite_type_cost (campsite_type_id, season_id, cost_per_night, min_nights) values (16, 27, 1, 0) $$,
	'23514', 'new row for relation "campsite_type_cost" violates check constraint "at_least_one_night"',
	'Should not be able to insert campsite type costs with at a night stay.'
);

reset role;


select *
from finish();

rollback;

