-- Test campsite_type_option
set client_min_messages to warning;
create extension if not exists pgtap;
reset client_min_messages;

begin;

select plan(40);

set search_path to camper, public;

select has_table('campsite_type_option');
select has_pk('campsite_type_option');
select table_privs_are('campsite_type_option', 'guest', array['SELECT']);
select table_privs_are('campsite_type_option', 'employee', array['SELECT']);
select table_privs_are('campsite_type_option', 'admin', array['SELECT', 'INSERT', 'UPDATE', 'DELETE']);
select table_privs_are('campsite_type_option', 'authenticator', array[]::text[]);

select has_column('campsite_type_option', 'campsite_type_option_id');
select col_is_pk('campsite_type_option', 'campsite_type_option_id');
select col_type_is('campsite_type_option', 'campsite_type_option_id', 'integer');
select col_not_null('campsite_type_option', 'campsite_type_option_id');
select col_hasnt_default('campsite_type_option', 'campsite_type_option_id');

select has_column('campsite_type_option', 'campsite_type_id');
select col_is_fk('campsite_type_option', 'campsite_type_id');
select fk_ok('campsite_type_option', 'campsite_type_id', 'campsite_type', 'campsite_type_id');
select col_type_is('campsite_type_option', 'campsite_type_id', 'integer');
select col_not_null('campsite_type_option', 'campsite_type_id');
select col_hasnt_default('campsite_type_option', 'campsite_type_id');

select has_column('campsite_type_option', 'name');
select col_type_is('campsite_type_option', 'name', 'text');
select col_not_null('campsite_type_option', 'name');
select col_hasnt_default('campsite_type_option', 'name');

select has_column('campsite_type_option', 'range');
select col_type_is('campsite_type_option', 'range', 'int4range');
select col_not_null('campsite_type_option', 'range');
select col_hasnt_default('campsite_type_option', 'range');


set client_min_messages to warning;
truncate campsite_type_option cascade;
truncate campsite_type cascade;
truncate media cascade;
truncate media_content cascade;
truncate company_host cascade;
truncate company_user cascade;
truncate company cascade;
truncate auth."user" cascade;
reset client_min_messages;

insert into auth."user" (user_id, email, name, password, cookie, cookie_expires_at)
values (1, 'demo@tandem.blog', 'Demo', 'test', '44facbb30d8a419dfd4bfbc44a4b5539d4970148dfc84bed0e', current_timestamp + interval '1 month')
     , (5, 'admin@tandem.blog', 'Demo', 'test', '12af4c88b528c2ad4222e3740496ecbc58e76e26f087657524', current_timestamp + interval '1 month')
;

insert into company (company_id, business_name, vatin, trade_name, phone, email, web, address, city, province, postal_code, country_code, currency_code, default_lang_tag)
values (2, 'Company 2', 'XX123', '', '555-555-555', 'a@a', '', '', '', '', '', 'ES', 'EUR', 'ca')
     , (4, 'Company 4', 'XX234', '', '666-666-666', 'b@b', '', '', '', '', '', 'FR', 'USD', 'ca')
;

insert into company_user (company_id, user_id, role)
values (2, 1, 'admin')
     , (4, 5, 'admin')
;

insert into company_host (company_id, host)
values (2, 'co2')
     , (4, 'co4')
;

insert into media_content (media_type, bytes)
values ('image/x-xpixmap', 'static char *s[]={"1 1 1 1","a c #ffffff","a"};')
;

insert into media (media_id, company_id, original_filename, content_hash)
values (6, 2, 'cover2.xpm', sha256('static char *s[]={"1 1 1 1","a c #ffffff","a"};'))
     , (8, 4, 'cover4.xpm', sha256('static char *s[]={"1 1 1 1","a c #ffffff","a"};'))
;

insert into campsite_type (campsite_type_id, company_id, name, media_id, dogs_allowed, max_campers)
values (16, 2, 'Wooden lodge', 6, false, 7)
     , (18, 4, 'Bungalow', 8, false, 6)
;

insert into campsite_type_option (campsite_type_id, name, range)
values (16, 'Option 16.1', '[2, 2]')
     , (18, 'Option 18.1', '[4, 8]')
;

prepare campsite_option_data as
select campsite_type_id, name
from campsite_type_option
;

set role guest;
select bag_eq(
	'campsite_option_data',
	$$ values (16, 'Option 16.1')
	        , (18, 'Option 18.1')
	$$,
	'Everyone should be able to list all campsite type options across all companies'
);
reset role;

select set_cookie('44facbb30d8a419dfd4bfbc44a4b5539d4970148dfc84bed0e/demo@tandem.blog', 'co2');

select lives_ok(
	$$ insert into campsite_type_option(campsite_type_id, name, range) values (16, 'Option 16.2', '[3, 3]') $$,
	'Admin from company 2 should be able to insert a new campsite type option to that company.'
);

select bag_eq(
	'campsite_option_data',
	$$ values (16, 'Option 16.1')
	        , (16, 'Option 16.2')
	        , (18, 'Option 18.1')
	$$,
	'The new row should have been added'
);

select lives_ok(
	$$ update campsite_type_option set name = 'Option 16-2' where campsite_type_id = 16 and name = 'Option 16.2' $$,
	'Admin from company 2 should be able to update campsite type option of that company.'
);

select bag_eq(
	'campsite_option_data',
	$$ values (16, 'Option 16.1')
	        , (16, 'Option 16-2')
	        , (18, 'Option 18.1')
	$$,
	'The row should have been updated.'
);

select lives_ok(
	$$ delete from campsite_type_option where campsite_type_id = 16 and name = 'Option 16-2' $$,
	'Admin from company 2 should be able to delete campsite type option from that company.'
);

select bag_eq(
	'campsite_option_data',
	$$ values (16, 'Option 16.1')
	        , (18, 'Option 18.1')
	$$,
	'The row should have been deleted.'
);

select throws_ok(
	$$ insert into campsite_type_option (campsite_type_id, name, range) values (18, 'Option 18.2', '[5, 5]') $$,
	'42501', 'new row violates row-level security policy for table "campsite_type_option"',
	'Admin from company 2 should NOT be able to insert new campsite type options to company 4.'
);

select lives_ok(
	$$ update campsite_type_option set name = 'Option 18-1' where campsite_type_id = 18 $$,
	'Admin from company 2 should not be able to update campsite types of company 4, but no error if campsite_type_id is not changed.'
);

select bag_eq(
	'campsite_option_data',
	$$ values (16, 'Option 16.1')
	        , (18, 'Option 18.1')
	$$,
	'No row should have been changed.'
);

select throws_ok(
	$$ update campsite_type_option set campsite_type_id = 18 where campsite_type_id = 16 $$,
	'42501', 'new row violates row-level security policy for table "campsite_type_option"',
	'Admin from company 2 should NOT be able to move campsite type option to one of company 4'
);

select lives_ok(
	$$ delete from campsite_type_option where campsite_type_id = 18 $$,
	'Admin from company 2 should NOT be able to delete campsite type from company 4, but not error is thrown'
);

select bag_eq(
	'campsite_option_data',
	$$ values (16, 'Option 16.1')
	        , (18, 'Option 18.1')
	$$,
	'No row should have been changed'
);

select throws_ok(
	$$ insert into campsite_type_option (campsite_type_id, name, range) values (16, '  ', '[5, 6]') $$,
	'23514', 'new row for relation "campsite_type_option" violates check constraint "name_not_empty"',
	'Should not be able to insert campsite type options with a blank name.'
);

select throws_ok(
	$$ insert into campsite_type_option (campsite_type_id, name, range) values (16, 'Option 16.2', '[-1, 1]') $$,
	'23514', 'new row for relation "campsite_type_option" violates check constraint "range_not_negative"',
	'Should not be able to insert campsite type options with a range starting with a negative number'
);

reset role;


select *
from finish();

rollback;

