-- Test edit_media
set client_min_messages to warning;
create extension if not exists pgtap;
reset client_min_messages;

begin;

select plan(13);

set search_path to camper, public;

select has_function('camper', 'edit_media', array['integer', 'text', 'media_type', 'bytea']);
select function_lang_is('camper', 'edit_media', array['integer', 'text', 'media_type', 'bytea'], 'plpgsql');
select function_returns('camper', 'edit_media', array['integer', 'text', 'media_type', 'bytea'], 'integer');
select isnt_definer('camper', 'edit_media', array['integer', 'text', 'media_type', 'bytea']);
select volatility_is('camper', 'edit_media', array['integer', 'text', 'media_type', 'bytea'], 'volatile');
select function_privs_are('camper', 'edit_media', array ['integer', 'text', 'media_type', 'bytea'], 'guest', array[]::text[]);
select function_privs_are('camper', 'edit_media', array ['integer', 'text', 'media_type', 'bytea'], 'employee', array[]::text[]);
select function_privs_are('camper', 'edit_media', array ['integer', 'text', 'media_type', 'bytea'], 'admin', array['EXECUTE']);
select function_privs_are('camper', 'edit_media', array ['integer', 'text', 'media_type', 'bytea'], 'authenticator', array[]::text[]);


set client_min_messages to warning;
truncate media cascade;
truncate media_content cascade;
truncate company cascade;
reset client_min_messages;

insert into company (company_id, business_name, vatin, trade_name, phone, email, web, address, city, province, postal_code, country_code, currency_code, default_lang_tag)
values (1, 'Company 2', 'XX123', '', '555-555-555', 'a@a', '', '', '', '', '', 'ES', 'EUR', 'ca')
;

insert into media_content (media_type, bytes)
values ('image/x-xpixmap', 'static char *s[]={"1 1 1 1","a c #ffffff","a"};')
     , ('image/x-xpixmap', 'static char *s[]={"1 1 1 1","a c #ff00ff","a"};')
;

insert into media (media_id, company_id, original_filename, content_hash)
values (2, 1, 'cover2.xpm', sha256('static char *s[]={"1 1 1 1","a c #ffffff","a"};'))
     , (3, 1, 'cover3.xpm', sha256('static char *s[]={"1 1 1 1","a c #ff00ff","a"};'))
     , (4, 1, 'cover4.xpm', sha256('static char *s[]={"1 1 1 1","a c #ff00ff","a"};'))
;

select lives_ok(
	$$ select edit_media(2, 'cover_2.xpm') $$,
	'Should be able to change the filename of the first media'
);

select lives_ok(
	$$ select edit_media(3, 'cover_3.xpm', 'image/x-xpixmap', 'static char *s[]={"1 1 1 1","a c #ffffff","a"};') $$,
	'Should be able to change the filename and hash of the second media to point to a different content'
);

select lives_ok(
	$$ select edit_media(4, 'cover4.svg', 'image/svg+xml', '<svg xmlns="http://www.w3.org/2000/svg" width="1" height="1"/>') $$,
	'Should be able to change the filename and hash of the thirs media to point to a new content'
);

select bag_eq(
	$$ select media_id, content_hash, original_filename, media_type, convert_from(bytes, 'utf-8') from media join media_content using (content_hash) $$,
	$$ values (2, sha256('static char *s[]={"1 1 1 1","a c #ffffff","a"};'), 'cover_2.xpm', 'image/x-xpixmap', 'static char *s[]={"1 1 1 1","a c #ffffff","a"};')
	        , (3, sha256('static char *s[]={"1 1 1 1","a c #ffffff","a"};'), 'cover_3.xpm', 'image/x-xpixmap', 'static char *s[]={"1 1 1 1","a c #ffffff","a"};')
	        , (4, sha256('<svg xmlns="http://www.w3.org/2000/svg" width="1" height="1"/>'), 'cover4.svg', 'image/svg+xml', '<svg xmlns="http://www.w3.org/2000/svg" width="1" height="1"/>')
	$$,
	'Should have added all three media'
);

select *
from finish();

rollback;
